@@ -29,6 +29,10 @@ public enum OrcConf {
   STRIPE_SIZE("orc.stripe.size", "hive.exec.orc.default.stripe.size",
       64L * 1024 * 1024,
       "Define the default ORC stripe size, in bytes."),
+  STRIPE_ROW_COUNT("orc.stripe.row.count", "orc.stripe.row.count",
+      Integer.MAX_VALUE, "This value limit the row count in one stripe. \n" +
+      "The number of stripe rows can be controlled at \n" +
+      "(0, \"orc.stripe.row.count\" + max(batchSize, \"orc.rows.between.memory.checks\"))"),
   BLOCK_SIZE("orc.block.size", "hive.exec.orc.default.block.size",
       256L * 1024 * 1024,
       "Define the default file system block size for ORC files."),
@@ -147,7 +151,7 @@ public enum OrcConf {
       "The kryo and base64 encoded SearchArgument for predicate pushdown."),
   KRYO_SARG_BUFFER("orc.kryo.sarg.buffer", null, 8192,
       "The kryo buffer size for SearchArgument for predicate pushdown."),
-  SARG_COLUMNS("orc.sarg.column.names", "orc.sarg.column.names", null,
+  SARG_COLUMNS("orc.sarg.column.names", "org.sarg.column.names", null,
       "The list of column names for the SearchArgument."),
   FORCE_POSITIONAL_EVOLUTION("orc.force.positional.evolution",
       "orc.force.positional.evolution", false,
@@ -161,14 +165,15 @@ public enum OrcConf {
   ROWS_BETWEEN_CHECKS("orc.rows.between.memory.checks", "orc.rows.between.memory.checks", 5000,
     "How often should MemoryManager check the memory sizes? Measured in rows\n" +
       "added to all of the writers.  Valid range is [1,10000] and is primarily meant for" +
-      "testing.  Setting this too low may negatively affect performance."),
+      "testing.  Setting this too low may negatively affect performance."
+        + " Use orc.stripe.row.count instead if the value larger than orc.stripe.row.count."),
   OVERWRITE_OUTPUT_FILE("orc.overwrite.output.file", "orc.overwrite.output.file", false,
     "A boolean flag to enable overwriting of the output file if it already exists.\n"),
   IS_SCHEMA_EVOLUTION_CASE_SENSITIVE("orc.schema.evolution.case.sensitive",
       "orc.schema.evolution.case.sensitive", true,
       "A boolean flag to determine if the comparision of field names " +
       "in schema evolution is case sensitive .\n"),
-  ALLOW_SARG_TO_FILTER("orc.sarg.to.filter", "orc.sarg.to.filter", false,
+  ALLOW_SARG_TO_FILTER("orc.sarg.to.filter", "org.sarg.to.filter", false,
                        "A boolean flag to determine if a SArg is allowed to become a filter"),
   READER_USE_SELECTED("orc.filter.use.selected", "orc.filter.use.selected", false,
                         "A boolean flag to determine if the selected vector is supported by\n"
@@ -176,6 +181,15 @@ public enum OrcConf {
                         + "must have the filter\n"
                         + "reapplied to avoid using unset values in the unselected rows.\n"
                         + "If unsure please leave this as false."),
+  ALLOW_PLUGIN_FILTER("orc.filter.plugin",
+                      "orc.filter.plugin",
+                      false,
+                      "Enables the use of plugin filters during read. The plugin filters "
+                      + "are discovered against the service "
+                      + "org.apache.orc.filter.PluginFilterService, if multiple filters are "
+                      + "determined, they are combined using AND. The order of application is "
+                      + "non-deterministic and the filter functionality should not depend on the "
+                      + "order of application."),
   WRITE_VARIABLE_LENGTH_BLOCKS("orc.write.variable.length.blocks", null, false,
       "A boolean flag as to whether the ORC writer should write variable length\n"
       + "HDFS blocks."),
@@ -185,6 +199,19 @@ public enum OrcConf {
   ORC_MAX_DISK_RANGE_CHUNK_LIMIT("orc.max.disk.range.chunk.limit",
       "hive.exec.orc.max.disk.range.chunk.limit",
     Integer.MAX_VALUE - 1024, "When reading stripes >2GB, specify max limit for the chunk size."),
+  ORC_MIN_DISK_SEEK_SIZE("orc.min.disk.seek.size",
+                         "orc.min.disk.seek.size",
+                         0,
+                         "When determining contiguous reads, gaps within this size are "
+                         + "read contiguously and not seeked. Default value of zero disables this "
+                         + "optimization"),
+  ORC_MIN_DISK_SEEK_SIZE_TOLERANCE("orc.min.disk.seek.size.tolerance",
+                          "orc.min.disk.seek.size.tolerance", 0.00,
+                          "Define the tolerance for for extra bytes read as a result of "
+                          + "orc.min.disk.seek.size. If the "
+                          + "(bytesRead - bytesNeeded) / bytesNeeded is greater than this "
+                          + "threshold then extra work is performed to drop the extra bytes from "
+                          + "memory after the read."),
   ENCRYPTION("orc.encrypt", "orc.encrypt", null, "The list of keys and columns to encrypt with"),
   DATA_MASK("orc.mask", "orc.mask", null, "The masks to apply to the encrypted columns"),
   KEY_PROVIDER("orc.key.provider", "orc.key.provider", "hadoop",
