@@ -79,6 +79,12 @@ import java.util.function.Consumer;
 public class RecordReaderImpl implements RecordReader {
   static final Logger LOG = LoggerFactory.getLogger(RecordReaderImpl.class);
   private static final boolean isLogDebugEnabled = LOG.isDebugEnabled();
+  // as public for use with test cases
+  public static final OrcProto.ColumnStatistics EMPTY_COLUMN_STATISTICS =
+      OrcProto.ColumnStatistics.newBuilder().setNumberOfValues(0)
+          .setHasNull(false)
+          .setBytesOnDisk(0)
+          .build();
   protected final Path path;
   private final long firstRow;
   private final List<StripeInformation> stripes = new ArrayList<>();
@@ -132,7 +138,7 @@ public class RecordReaderImpl implements RecordReader {
   static TypeDescription findColumnType(SchemaEvolution evolution, String columnName) {
     try {
       TypeDescription readerColumn = evolution.getReaderBaseSchema().findSubtype(
-        columnName, evolution.isSchemaEvolutionCaseAware);
+          columnName, evolution.isSchemaEvolutionCaseAware);
       return evolution.getFileType(readerColumn);
     } catch (IllegalArgumentException e) {
       throw new IllegalArgumentException("Filter could not find column with name: " +
@@ -154,7 +160,7 @@ public class RecordReaderImpl implements RecordReader {
   static TypeDescription findMostCommonColumn(SchemaEvolution evolution, String columnName) {
     try {
       TypeDescription readerColumn = evolution.getReaderBaseSchema().findSubtype(
-        columnName, evolution.isSchemaEvolutionCaseAware);
+          columnName, evolution.isSchemaEvolutionCaseAware);
       TypeDescription fileColumn;
       do {
         fileColumn = evolution.getFileType(readerColumn);
@@ -231,7 +237,7 @@ public class RecordReaderImpl implements RecordReader {
     this.fileIncluded = evolution.getFileIncluded();
     SearchArgument sarg = options.getSearchArgument();
     boolean[] rowIndexCols = new boolean[evolution.getFileIncluded().length];
-    if (sarg != null && rowIndexStride > 0) {
+    if (sarg != null && rowIndexStride != 0) {
       sargApp = new SargApplier(sarg,
           rowIndexStride,
           evolution,
@@ -275,7 +281,9 @@ public class RecordReaderImpl implements RecordReader {
               .withFileSystemSupplier(fileReader.getFileSystemSupplier())
               .withPath(fileReader.path)
               .withMaxDiskRangeChunkLimit(maxDiskRangeChunkLimit)
-              .withZeroCopy(zeroCopy);
+              .withZeroCopy(zeroCopy)
+              .withMinSeekSize(options.minSeekSize())
+              .withMinSeekSizeTolerance(options.minSeekSizeTolerance());
       FSDataInputStream file = fileReader.takeFile();
       if (file != null) {
         builder.withFile(file);
@@ -292,11 +300,15 @@ public class RecordReaderImpl implements RecordReader {
 
     String[] filterCols = null;
     Consumer<OrcFilterContext> filterCallBack = null;
+    String filePath = options.allowPluginFilters() ?
+        fileReader.getFileSystem().makeQualified(fileReader.path).toString() : null;
     BatchFilter filter = FilterFactory.createBatchFilter(options,
                                                          evolution.getReaderBaseSchema(),
                                                          evolution.isSchemaEvolutionCaseAware(),
                                                          fileReader.getFileVersion(),
-                                                         false);
+                                                         false,
+                                                         filePath,
+                                                         fileReader.conf);
     if (filter != null) {
       // If a filter is determined then use this
       filterCallBack = filter;
@@ -430,18 +442,33 @@ public class RecordReaderImpl implements RecordReader {
     final boolean onlyLowerBound;
     final boolean onlyUpperBound;
     final boolean hasNulls;
+    final boolean hasValue;
+    final boolean comparable;
 
     ValueRange(PredicateLeaf predicate,
                T lower, T upper,
                boolean hasNulls,
                boolean onlyLowerBound,
-               boolean onlyUpperBound) {
+               boolean onlyUpperBound,
+               boolean hasValue,
+               boolean comparable) {
       PredicateLeaf.Type type = predicate.getType();
       this.lower = getBaseObjectForComparison(type, lower);
       this.upper = getBaseObjectForComparison(type, upper);
       this.hasNulls = hasNulls;
       this.onlyLowerBound = onlyLowerBound;
       this.onlyUpperBound = onlyUpperBound;
+      this.hasValue = hasValue;
+      this.comparable = comparable;
+    }
+
+    ValueRange(PredicateLeaf predicate,
+               T lower, T upper,
+               boolean hasNulls,
+               boolean onlyLowerBound,
+               boolean onlyUpperBound) {
+      this(predicate, lower, upper, hasNulls, onlyLowerBound, onlyUpperBound,
+          lower != null, lower != null);
     }
 
     ValueRange(PredicateLeaf predicate, T lower, T upper,
@@ -459,7 +486,25 @@ public class RecordReaderImpl implements RecordReader {
     }
 
     boolean hasValues() {
-      return lower != null;
+      return hasValue;
+    }
+
+    /**
+     * Whether min or max is provided for comparison
+     * @return is it comparable
+     */
+    boolean isComparable() {
+      return hasValue && comparable;
+    }
+
+    /**
+     * value range is invalid if the column statistics are non-existent
+     * @see ColumnStatisticsImpl#isStatsExists()
+     * this method is similar to isStatsExists
+     * @return value range is valid or not
+     */
+    boolean isValid() {
+      return hasValue || hasNulls;
     }
 
     /**
@@ -501,14 +546,14 @@ public class RecordReaderImpl implements RecordReader {
     TruthValue addNull(TruthValue value) {
       if (hasNulls) {
         switch (value) {
-        case YES:
-          return TruthValue.YES_NULL;
-        case NO:
-          return TruthValue.NO_NULL;
-        case YES_NO:
-          return TruthValue.YES_NO_NULL;
-        default:
-          return value;
+          case YES:
+            return TruthValue.YES_NULL;
+          case NO:
+            return TruthValue.NO_NULL;
+          case YES_NO:
+            return TruthValue.YES_NO_NULL;
+          default:
+            return value;
         }
       } else {
         return value;
@@ -571,7 +616,7 @@ public class RecordReaderImpl implements RecordReader {
       Boolean max = stats.getTrueCount() != 0;
       return new ValueRange<>(predicate, min, max, stats.hasNull());
     } else {
-      return new ValueRange(predicate, null, null, true);
+      return new ValueRange(predicate, null, null, index.hasNull(), false, false, true, false);
     }
   }
 
@@ -650,8 +695,8 @@ public class RecordReaderImpl implements RecordReader {
                    " include ORC-517. Writer version: {}",
           predicate.getColumnName(), writerVersion);
       return TruthValue.YES_NO_NULL;
-    } else if ((category == TypeDescription.Category.DOUBLE ||
-        category == TypeDescription.Category.FLOAT) && cs instanceof DoubleColumnStatistics) {
+    } else if (category == TypeDescription.Category.DOUBLE
+        || category == TypeDescription.Category.FLOAT) {
       DoubleColumnStatistics dstas = (DoubleColumnStatistics) cs;
       if (Double.isNaN(dstas.getSum())) {
         LOG.debug("Not using predication pushdown on {} because stats contain NaN values",
@@ -703,6 +748,10 @@ public class RecordReaderImpl implements RecordReader {
                                            ValueRange range,
                                            BloomFilter bloomFilter,
                                            boolean useUTCTimestamp) {
+    if (!range.isValid()) {
+      return TruthValue.YES_NO_NULL;
+    }
+
     // if we didn't have any values, everything must have been null
     if (!range.hasValues()) {
       if (predicate.getOperator() == PredicateLeaf.Operator.IS_NULL) {
@@ -710,6 +759,8 @@ public class RecordReaderImpl implements RecordReader {
       } else {
         return TruthValue.NULL;
       }
+    } else if (!range.isComparable()) {
+      return range.hasNulls ? TruthValue.YES_NO_NULL : TruthValue.YES_NO;
     }
 
     TruthValue result;
@@ -896,13 +947,13 @@ public class RecordReaderImpl implements RecordReader {
         result = TruthValue.YES_NO_NULL;
       }
     } else {
-        // if the predicate object is null and if hasNull says there are no nulls then return NO
-        if (predObj == null && !hasNull) {
-          result = TruthValue.NO;
-        } else {
-          result = TruthValue.YES_NO_NULL;
-        }
+      // if the predicate object is null and if hasNull says there are no nulls then return NO
+      if (predObj == null && !hasNull) {
+        result = TruthValue.NO;
+      } else {
+        result = TruthValue.YES_NO_NULL;
       }
+    }
 
     if (result == TruthValue.YES_NO_NULL && !hasNull) {
       result = TruthValue.YES_NO;
@@ -1131,7 +1182,10 @@ public class RecordReaderImpl implements RecordReader {
             if (entry == null) {
               throw new AssertionError("RG is not populated for " + columnIx + " rg " + rowGroup);
             }
-            OrcProto.ColumnStatistics stats = entry.getStatistics();
+            OrcProto.ColumnStatistics stats = EMPTY_COLUMN_STATISTICS;
+            if (entry.hasStatistics()) {
+              stats = entry.getStatistics();
+            }
             OrcProto.BloomFilter bf = null;
             OrcProto.Stream.Kind bfk = null;
             if (bloomFilterIndices != null && bloomFilterIndices[columnIx] != null) {
@@ -1654,12 +1708,4 @@ public class RecordReaderImpl implements RecordReader {
   public int getMaxDiskRangeChunkLimit() {
     return maxDiskRangeChunkLimit;
   }
-
-  /**
-   * Get sargApplier for testing.
-   * @return sargApplier in record reader.
-   */
-  SargApplier getSargApp() {
-    return sargApp;
-  }
 }
