@@ -69,13 +69,13 @@ import java.util.TreeMap;
  * type of column. TreeWriters may have children TreeWriters that handle the
  * sub-types. Each of the TreeWriters writes the column's data as a set of
  * streams.
- *
+ * <p>
  * This class is unsynchronized like most Stream objects, so from the creation
  * of an OrcFile and all access to a single instance has to be from a single
  * thread.
- *
+ * <p>
  * There are no known cases where these happen between different threads today.
- *
+ * <p>
  * Caveat: the MemoryManager is created during WriterOptions create, that has
  * to be confined to a single thread as well.
  *
@@ -88,6 +88,7 @@ public class WriterImpl implements WriterInternal, MemoryManager.Callback {
 
   private final Path path;
   private final long stripeSize;
+  private final long stripeRowCount;
   private final int rowIndexStride;
   private final TypeDescription schema;
   private final PhysicalWriter physicalWriter;
@@ -101,9 +102,9 @@ public class WriterImpl implements WriterInternal, MemoryManager.Callback {
   private long lastFlushOffset = 0;
   private int stripesAtLastFlush = -1;
   private final List<OrcProto.StripeInformation> stripes =
-    new ArrayList<>();
+      new ArrayList<>();
   private final Map<String, ByteString> userMetadata =
-    new TreeMap<>();
+      new TreeMap<>();
   private final TreeWriter treeWriter;
   private final boolean buildIndex;
   private final MemoryManager memoryManager;
@@ -182,14 +183,9 @@ public class WriterImpl implements WriterInternal, MemoryManager.Callback {
     this.encodingStrategy = opts.getEncodingStrategy();
     this.compressionStrategy = opts.getCompressionStrategy();
 
-    if (opts.getRowIndexStride() >= 0) {
-      this.rowIndexStride = opts.getRowIndexStride();
-    } else {
-      this.rowIndexStride = 0;
-    }
+    this.rowIndexStride = opts.getRowIndexStride();
 
-    // ORC-1343: We ignore `opts.isBuildIndex` due to the lack of reader support
-    this.buildIndex = rowIndexStride > 0;
+    this.buildIndex = opts.isBuildIndex() && (rowIndexStride > 0);
     if (buildIndex && rowIndexStride < MIN_ROW_INDEX_STRIDE) {
       throw new IllegalArgumentException("Row stride must be at least " +
           MIN_ROW_INDEX_STRIDE);
@@ -216,7 +212,9 @@ public class WriterImpl implements WriterInternal, MemoryManager.Callback {
     }
 
     // ensure that we are able to handle callbacks before we register ourselves
-    ROWS_PER_CHECK = OrcConf.ROWS_BETWEEN_CHECKS.getLong(conf);
+    ROWS_PER_CHECK = Math.min(opts.getStripeRowCountValue(),
+        OrcConf.ROWS_BETWEEN_CHECKS.getLong(conf));
+    this.stripeRowCount= opts.getStripeRowCountValue();
     this.stripeSize = opts.getStripeSize();
     memoryLimit = stripeSize;
     memoryManager = opts.getMemoryManager();
@@ -304,9 +302,10 @@ public class WriterImpl implements WriterInternal, MemoryManager.Callback {
       long size = treeWriter.estimateMemory();
       if (LOG.isDebugEnabled()) {
         LOG.debug("ORC writer " + physicalWriter + " size = " + size +
-                      " limit = " + memoryLimit);
+            " memoryLimit = " + memoryLimit + " rowsInStripe = " + rowsInStripe +
+            " stripeRowCountLimit = " + stripeRowCount);
       }
-      if (size > memoryLimit) {
+      if (size > memoryLimit || rowsInStripe >= stripeRowCount) {
         flushStripe();
         return true;
       }
@@ -666,7 +665,7 @@ public class WriterImpl implements WriterInternal, MemoryManager.Callback {
     // add all of the user metadata
     for(Map.Entry<String, ByteString> entry: userMetadata.entrySet()) {
       builder.addMetadata(OrcProto.UserMetadataItem.newBuilder()
-        .setName(entry.getKey()).setValue(entry.getValue()));
+          .setName(entry.getKey()).setValue(entry.getValue()));
     }
     if (encryption.length > 0) {
       builder.setEncryption(writeEncryptionFooter());
@@ -702,7 +701,18 @@ public class WriterImpl implements WriterInternal, MemoryManager.Callback {
         while (posn < batch.size) {
           int chunkSize = Math.min(batch.size - posn,
               rowIndexStride - rowsInIndex);
-          treeWriter.writeRootBatch(batch, posn, chunkSize);
+          if (batch.isSelectedInUse()) {
+            // find the longest chunk that is continuously selected from posn
+            for (int len = 1; len < chunkSize; ++len) {
+              if (batch.selected[posn + len] - batch.selected[posn] != len) {
+                chunkSize = len;
+                break;
+              }
+            }
+            treeWriter.writeRootBatch(batch, batch.selected[posn], chunkSize);
+          } else {
+            treeWriter.writeRootBatch(batch, posn, chunkSize);
+          }
           posn += chunkSize;
           rowsInIndex += chunkSize;
           rowsInStripe += chunkSize;
@@ -711,8 +721,24 @@ public class WriterImpl implements WriterInternal, MemoryManager.Callback {
           }
         }
       } else {
+        if (batch.isSelectedInUse()) {
+          int posn = 0;
+          while (posn < batch.size) {
+            int chunkSize = 1;
+            while (posn + chunkSize < batch.size) {
+              // find the longest chunk that is continuously selected from posn
+              if (batch.selected[posn + chunkSize] - batch.selected[posn] != chunkSize) {
+                break;
+              }
+              ++chunkSize;
+            }
+            treeWriter.writeRootBatch(batch, batch.selected[posn], chunkSize);
+            posn += chunkSize;
+          }
+        } else {
+          treeWriter.writeRootBatch(batch, 0, batch.size);
+        }
         rowsInStripe += batch.size;
-        treeWriter.writeRootBatch(batch, 0, batch.size);
       }
       rowsSinceCheck += batch.size;
       previousAllocation = memoryManager.checkMemory(previousAllocation, this);
