@@ -15,20 +15,6 @@
  */
 package org.apache.orc.impl;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertTrue;
-
-import java.io.ByteArrayInputStream;
-import java.io.EOFException;
-import java.io.IOException;
-import java.net.URI;
-import java.net.URISyntaxException;
-import java.nio.ByteBuffer;
-import java.nio.charset.CharacterCodingException;
-import java.nio.charset.StandardCharsets;
-import java.util.ArrayList;
-import java.util.List;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FSDataInputStream;
 import org.apache.hadoop.fs.FSDataOutputStream;
@@ -39,6 +25,8 @@ import org.apache.hadoop.fs.PositionedReadable;
 import org.apache.hadoop.fs.Seekable;
 import org.apache.hadoop.fs.permission.FsPermission;
 import org.apache.hadoop.hive.ql.exec.vector.VectorizedRowBatch;
+import org.apache.hadoop.hive.ql.exec.vector.LongColumnVector;
+import org.apache.hadoop.hive.ql.exec.vector.Decimal64ColumnVector;
 import org.apache.hadoop.hive.ql.io.sarg.PredicateLeaf;
 import org.apache.hadoop.hive.ql.io.sarg.SearchArgument;
 import org.apache.hadoop.hive.ql.io.sarg.SearchArgumentFactory;
@@ -53,24 +41,35 @@ import org.apache.orc.RecordReader;
 import org.apache.orc.StripeStatistics;
 import org.apache.orc.TestVectorOrcFile;
 import org.apache.orc.TypeDescription;
-import org.junit.Before;
-import org.junit.Rule;
-import org.junit.Test;
-import org.junit.rules.ExpectedException;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+
+import java.io.ByteArrayInputStream;
+import java.io.EOFException;
+import java.io.IOException;
+import java.net.URI;
+import java.net.URISyntaxException;
+import java.nio.ByteBuffer;
+import java.nio.charset.CharacterCodingException;
+import java.nio.charset.StandardCharsets;
+import java.util.ArrayList;
+import java.util.List;
+
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertThrows;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 public class TestReaderImpl {
   private Path workDir = new Path(System.getProperty("example.dir",
       "../../examples/"));
 
-  @Rule
-  public ExpectedException thrown = ExpectedException.none();
-
   private final Path path = new Path("test-file.orc");
   private FSDataInputStream in;
   private int psLen;
   private ByteBuffer buffer;
 
-  @Before
+  @BeforeEach
   public void setup() {
     in = null;
   }
@@ -78,15 +77,17 @@ public class TestReaderImpl {
   @Test
   public void testEnsureOrcFooterSmallTextFile() throws IOException {
     prepareTestCase("1".getBytes(StandardCharsets.UTF_8));
-    thrown.expect(FileFormatException.class);
-    ReaderImpl.ensureOrcFooter(in, path, psLen, buffer);
+    assertThrows(FileFormatException.class, () -> {
+      ReaderImpl.ensureOrcFooter(in, path, psLen, buffer);
+    });
   }
 
   @Test
   public void testEnsureOrcFooterLargeTextFile() throws IOException {
     prepareTestCase("This is Some Text File".getBytes(StandardCharsets.UTF_8));
-    thrown.expect(FileFormatException.class);
-    ReaderImpl.ensureOrcFooter(in, path, psLen, buffer);
+    assertThrows(FileFormatException.class, () -> {
+      ReaderImpl.ensureOrcFooter(in, path, psLen, buffer);
+    });
   }
 
   @Test
@@ -434,7 +435,7 @@ public class TestReaderImpl {
       TypeDescription schema = reader.getSchema();
       VectorizedRowBatch batch = schema.createRowBatch();
       try (RecordReader rows = reader.rows(opt)) {
-        assertTrue("No rows read out!", rows.nextBatch(batch));
+        assertTrue(rows.nextBatch(batch), "No rows read out!");
         assertEquals(5, batch.size);
         assertFalse(rows.nextBatch(batch));
       }
@@ -447,6 +448,45 @@ public class TestReaderImpl {
     CheckFileWithSargs("bad_bloom_filter_1.6.0.orc", "ORC C++ ");
   }
 
+  @Test
+  public void testReadDecimalV2File() throws IOException {
+    Configuration conf = new Configuration();
+    Path path = new Path(workDir, "decimal64_v2_cplusplus.orc");
+    FileSystem fs = path.getFileSystem(conf);
+    try (ReaderImpl reader = (ReaderImpl) OrcFile.createReader(path,
+        OrcFile.readerOptions(conf).filesystem(fs))) {
+      assertEquals("ORC C++ 1.8.0-SNAPSHOT", reader.getSoftwareVersion());
+      OrcTail tail = reader.extractFileTail(fs, path, Long.MAX_VALUE);
+      List<StripeStatistics> stats = tail.getStripeStatistics();
+      assertEquals(1, stats.size());
+
+      try (RecordReader rows = reader.rows()) {
+        TypeDescription schema = reader.getSchema();
+        assertEquals("struct<a:bigint,b:decimal(10,2),c:decimal(2,2),d:decimal(2,2),e:decimal(2,2)>",
+            schema.toString());
+        VectorizedRowBatch batch = schema.createRowBatchV2();
+        assertTrue(rows.nextBatch(batch), "No rows read out!");
+        assertEquals(10, batch.size);
+        LongColumnVector col1 = (LongColumnVector) batch.cols[0];
+        Decimal64ColumnVector col2 = (Decimal64ColumnVector) batch.cols[1];
+        Decimal64ColumnVector col3 = (Decimal64ColumnVector) batch.cols[2];
+        Decimal64ColumnVector col4 = (Decimal64ColumnVector) batch.cols[3];
+        Decimal64ColumnVector col5 = (Decimal64ColumnVector) batch.cols[4];
+        for (int i = 0; i < batch.size; ++i) {
+          assertEquals(17292380420L + i, col1.vector[i]);
+          if (i == 0) {
+            long scaleNum = (long) Math.pow(10, col2.scale);
+            assertEquals(164.16 * scaleNum, col2.vector[i]);
+          } else {
+            assertEquals(col2.vector[i - 1] * 2, col2.vector[i]);
+          }
+          assertEquals(col3.vector[i] + col4.vector[i], col5.vector[i]);
+        }
+        assertFalse(rows.nextBatch(batch));
+      }
+    }
+  }
+
   @Test
   public void testExtractFileTailIndexOutOfBoundsException() throws Exception {
     Configuration conf = new Configuration();
